% This script generates a 3D shape and a 2D projections
% of this shape
% 
% The user can change the following parameters:
%   nFeatures - Number of object's points
%   auxFeatures - Number of artifcial points
%   nImages1 - Number of images of set 1. This set allows us to compute the basis shape
%   nImages2 - Number of images of set 2
%   noiseSigma - Power of noise
%   usedNorm - The used norm
% 
% 
% For details, see:
% 
%    Marques, M., Stoic, M. and Costeira, J.. "Subspace matching: Unique solutions to point matching with geometric constraints",
%    ICCV2009 - 12th IEEE International Conference on Computer Vision, Kyoto, Japan, September 2009.
            
close all
clear all 

nFeatures = 12; % Number of object's points
auxFeatures = 2; % Number of articial points
nImages1 = 4; %Number of images of set 1. This set allows us to compute the basis shape
nImages2 = 3; %Number of images of set 2
noiseSigma = 0; % Power of noise
usedNorm = 2;

S = [randn(1,auxFeatures) zeros(1,nFeatures); zeros(3,auxFeatures) randn(3,nFeatures);
ones(1,nFeatures+auxFeatures)]; % Shape


W1 = [ ]; % Set 1
for i = 1:nImages1
[U,D,V] = svd(randn(3,2),'econ');
W1 = [W1; (rand+1)*[randn(2,1) U' randn(2,1)]*S];
end

W2 = []; [U,D,V] = svd(randn(3,2),'econ');% Set 2
for i = 1:nImages2
W2 = [W2; (rand+1)*[randn(2,1) U' randn(2,1)]*S];
end

vectPerm = randperm(nFeatures-1);
vectPerm =[1:(auxFeatures+1) vectPerm+(auxFeatures+1)];
Po = eye(length(W2)); Po = Po(:,vectPerm);

% add noise
corner = min(W2,[ ],2)*ones(1,nFeatures+auxFeatures);
window = (max(W2,[ ],2)-min(W2,[ ],2))*ones(1,nFeatures+auxFeatures);
noise = randn(2,nFeatures);
W2n = W2(:,vectPerm) + noiseSigma*window.*[zeros(2*nImages2,auxFeatures) repmat(noise,nImages2,1)];

P = subspaceMatching(W1,W2n,auxFeatures,usedNorm);
